/**
 * @ author: richen
 * @ copyright: Copyright (c) - <richenlin(at)gmail.com>
 * @ license: MIT
 * @ version: 2020-02-27 10:36:16
 */
declare const liteq: any;
interface AdapterInstance {
    knexClient: any;
    parser: any;
}
interface BaseModelInterface {
    pk: string;
    tableName: string;
    modelName: string;
    readonly getTableName: () => string;
    readonly getPk: () => string;
    readonly getInstance: (forceNew?: boolean) => Promise<Object>;
    readonly setInstance: (ins: AdapterInstance) => BaseModel;
    readonly field: (values: string | string[]) => BaseModel;
    readonly alias: (values: string) => BaseModel;
    readonly where: (values: Object) => BaseModel;
    readonly limit: (skip: number, limit?: number) => BaseModel;
    readonly order: (values: Object) => BaseModel;
    readonly distinct: (values: string[]) => BaseModel;
    readonly group: (values: string | string[]) => BaseModel;
    readonly having: (values: Object) => BaseModel;
    readonly join: (values: any[]) => BaseModel;
    readonly _beforeAdd: (data: Object, options?: Object) => Promise<Object>;
    readonly add: (data: Object | any[], options?: Object) => Promise<any>;
    readonly _afterAdd: (data: Object, options?: Object) => Promise<any>;
    readonly thenAdd: (data: Object, options?: Object) => Promise<any>;
    readonly _beforeDelete: (options: Object) => Promise<any>;
    readonly delete: (options: Object) => Promise<any>;
    readonly _afterDelete: (options: Object) => Promise<any>;
    readonly _beforeUpdate: (data: Object, options?: Object) => Promise<Object>;
    readonly update: (data: Object, options?: Object) => Promise<any>;
    readonly _afterUpdate: (data: Object, options?: Object) => Promise<any>;
    readonly increment: (field: string, step?: number, data?: Object, options?: Object) => Promise<any>;
    readonly decrement: (field: string, step?: number, data?: Object, options?: Object) => Promise<any>;
    readonly find: (options?: Object) => Promise<any>;
    readonly _afterFind: (result: any, options?: Object) => Promise<any>;
    readonly select: (options?: Object) => Promise<any[]>;
    readonly countSelect: (options?: Object) => Promise<any>;
    readonly _afterSelect: (result: any, options?: Object) => Promise<any>;
    readonly count: (field?: string, options?: Object) => Promise<any>;
    readonly sum: (field: string, options?: Object) => Promise<any>;
    readonly query: (sqlStr: string, params?: any[]) => Promise<any>;
    readonly transaction: (func: Function) => Promise<any>;
    readonly migrate: (sqlStr?: string) => Promise<any>;
    readonly sql: (options?: Object, data?: Object) => Promise<string>;
}
export declare class BaseModel extends liteq implements BaseModelInterface {
    tableName: string;
    modelName: string;
    pk: string;
    /**
     * Creates an instance of BaseModel.
     * @param {...any[]} args
     * @memberof BaseModel
     */
    constructor(...args: any[]);
    /**
     * Get table name
     *
     * @returns
     * @memberof BaseModel
     */
    getTableName(): string;
    /**
     * Get the primary key
     *
     * @returns {string}
     * @memberof BaseModel
     */
    getPk(): string;
    /**
     * Get repository handle
     *
     * @param {boolean} [forceNew=false]
     * @returns
     * @memberof BaseModel
     */
    getInstance(forceNew?: boolean): Promise<any>;
    /**
     * Set repository handle to model
     *
     * @param {AdapterInstance} ins
     * @returns
     * @memberof BaseModel
     */
    setInstance(ins: AdapterInstance): BaseModel;
    /**
     * Querying column names
     *
     * field(['aaa', 'bbb', 'ccc'])
     * @param {(string | string[])} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    field(values: string | string[]): BaseModel;
    /**
     * Querying table name alias
     *
     * @param {string} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    alias(values: string): BaseModel;
    /**
     * Query conditions
     *
     * or:  where({or: [{...}, {...}]})
     *
     * not: where({not: {name: '', id: 1}})
     *
     * notin: where({notin: {'id': [1,2,3]}})
     *
     * null: where({id: null})
     *
     * notnull: where({id: {"<>": null}})
     *
     * in: where({id: [1,2,3]})
     *
     * and: where({id: 1, name: 'a'},)
     *
     * operator: where({id: {'<>': 1}})
     *
     * operator: where({id: {'<>': 1, '>=': 0, '<': 100, '<=': 10}})
     *
     * like: where({name: {'like': '%a'}})
     *
     * @param {Object} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    where(values: Object): BaseModel;
    /**
     * Querying limit
     *
     * limit(1)
     *
     * limit(10, 20)
     *
     * @param {number} skip
     * @param {number} [limit]
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    limit(skip: number, limit?: number): BaseModel;
    /**
     * Querying order column
     *
     * order({xxx: 'desc'})
     *
     * @param {Object} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    order(values: Object): BaseModel;
    /**
     * Querying distinct columns
     *
     * distinct(['first_name'])
     *
     * @param {string[]} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    distinct(values: string[]): BaseModel;
    /**
     * Querying group columns
     *
     * group('xxx')
     *
     * group(['xxx', 'xxx'])
     *
     * @param {(string | string[])} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    group(values: string | string[]): BaseModel;
    /**
     * HAVING clause
     *
     * having({"name":{">": 100}})
     *
     * @param {Object} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    having(values: Object): BaseModel;
    /**
     * Join Querying
     *
     * join([{from: 'Test', alias: 'test', on: {aaa: bbb, ccc: ddd}, field: ['id', 'name'], type: 'inner'}])
     *
     * join([{from: 'Test', alias: 'test', on: {or: [{aaa: bbb}, {ccc: ddd}]}, field: ['id', 'name'], type: 'left'}])
     *
     * join([{from: 'Test', alias: 'test', on: {aaa: bbb, ccc: ddd}, field: ['id', 'name'], type: 'right'}])
     *
     * @param {any[]} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    join(values: any[]): BaseModel;
    /**
     * Pre-Add method
     *
     * @param {Object} data
     * @param {Object} [options]
     * @returns {Promise<Object>}
     * @memberof BaseModel
     */
    _beforeAdd(data: Object, options?: Object): Promise<Object>;
    /**
     * Add method
     *
     * @param {Object | any[]} data
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    add(data: Object | any[], options?: Object): Promise<any>;
    /**
     * Added after method
     *
     * @param {Object} data
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    _afterAdd(data: Object, options?: Object): Promise<any>;
    /**
     * Added when query result is empty
     *
     * @param {Object} data
     * @param {*} [options={ where: {} }]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    thenAdd(data: Object, options?: {
        where: {};
    }): Promise<any>;
    /**
     * Pre-Delete method
     *
     * @param {Object} options
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    _beforeDelete(options: Object): Promise<any>;
    /**
     * Delete method
     *
     * @param {*} [options={}]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    delete(options?: {}): Promise<any>;
    /**
     * Delete after method
     *
     * @param {Object} options
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    _afterDelete(options: Object): Promise<any>;
    /**
     * Pre-Update method
     *
     * @param {Object} data
     * @param {Object} [options]
     * @returns {Promise<Object>}
     * @memberof BaseModel
     */
    _beforeUpdate(data: Object, options?: Object): Promise<Object>;
    /**
     * Update method
     *
     * @param {Object} data
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    update(data: Object, options?: Object): Promise<any>;
    /**
     * Increment method
     *
     * @param {string} field
     * @param {number} [step=1]
     * @param {*} [data={}]
     * @param {Object} [options]
     * @returns
     * @memberof BaseModel
     */
    increment(field: string, step?: number, data?: {}, options?: Object): Promise<any>;
    /**
     * Decrement method
     *
     * @param {string} field
     * @param {number} [step=1]
     * @param {*} [data={}]
     * @param {Object} [options]
     * @returns
     * @memberof BaseModel
     */
    decrement(field: string, step?: number, data?: {}, options?: Object): Promise<any>;
    /**
     * Update after method
     *
     * @param {Object} data
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    _afterUpdate(data: Object, options?: Object): Promise<any>;
    /**
     * Find one
     *
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    find(options?: Object): Promise<any>;
    /**
     * Find one after method
     *
     * @param {*} result
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    _afterFind(result: any, options?: Object): Promise<any>;
    /**
     * Find all
     *
     * @param {Object} [options]
     * @returns {Promise<any[]>}
     * @memberof BaseModel
     */
    select(options?: Object): Promise<any[]>;
    /**
     * Paging query
     *
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    countSelect(options?: Object): Promise<any>;
    /**
     * Find all after method
     *
     * @param {*} result
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    _afterSelect(result: any, options?: Object): Promise<any>;
    /**
     * Count query
     *
     * @param {string} [field]
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    count(field?: string, options?: Object): Promise<any>;
    /**
     * Sum query
     *
     * @param {string} field
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    sum(field: string, options?: Object): Promise<any>;
    /**
     * Native statement query
     *
     * mysql/posgre  TestModel.query('select ?, ? from test where id=?', ['id', 'name', 1]);
     *
     * mongo  TestModel.query('db.test.find()');
     *
     * @param {string} sqlStr
     * @param {any[]} [params=[]]
     * @returns {Promise<any[]>}
     * @memberof BaseModel
     */
    query(sqlStr: string, params?: any[]): Promise<any[]>;
    /**
     * Execute transaction
     *
     * @param {Function} func
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    transaction(func: Function): Promise<any>;
    /**
     * Structure migration
     *
     * @param {string} [sqlStr]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    migrate(sqlStr?: string): Promise<any>;
    /**
     * sqlString build
     *
     * {method: find | select | add | update | count | sum | decrement | increment}
     *
     * @param {string} [options={ method: "select" }]
     * @param {*} [data={}]
     * @returns {Promise<string>}
     * @memberof BaseModel
     */
    sql(options?: {
        method: string;
    }, data?: {}): Promise<string>;
}
export {};
