/**
 * @ author: richen
 * @ copyright: Copyright (c) - <richenlin(at)gmail.com>
 * @ license: MIT
 * @ version: 2020-02-27 12:53:07
 */
import "reflect-metadata";
export { ClassValidator, FunctionValidator, IsDefined, IsCnName, IsIdNumber, IsZipCode, IsMobile, IsPlateNumber, IsEmail, IsIP, IsPhoneNumber, IsUrl, IsHash, IsNotEmpty, Equals, NotEquals, Contains, IsIn, IsNotIn, IsDate, Min, Max, Length } from "think_validtion";
/**
 * This decorator is used to mark classes that will be an entity (table or document depend on database type).
 * Database schema will be created for all classes decorated with it, and Repository can be retrieved and used for it.
 *
 * @export
 * @param {string} [identifier]
 * @returns {ClassDecorator}
 */
export declare function Entity(identifier?: string): ClassDecorator;
/**
 * Column decorator is used to mark a specific class property as a table column.
 * Only properties decorated with this decorator will be persisted to the database when entity be saved.
 * Primary columns also creates a PRIMARY KEY for this column in a db.
 *
 * @export
 * @param {number} [size=11]
 * @param {boolean} [auto=true]
 * @param {string} [comment]
 * @returns {PropertyDecorator}
 */
export declare function PrimaryColumn(size?: number, auto?: boolean, comment?: string): PropertyDecorator;
/**
 * Column decorator is used to mark a specific class property as a table column.
 *
 * @export
 * @param {number} [size]
 * @param {*} [defaultValue]
 * @param {boolean} [index=false]
 * @param {boolean} [unique=false]
 * @param {string} [comment]
 * @returns {PropertyDecorator}
 */
export declare function Column(size?: number, defaultValue?: any, index?: boolean, unique?: boolean, comment?: string): PropertyDecorator;
/**
 *
 */
export declare type timeWhen = "_beforeAdd" | "_beforeUpdate" | "All";
/**
 * This column will store a creation Unix timestamp of the inserted/updated object.
 *
 * @export
 * @param {timeWhen} [timeWhen="All"] When to execute timestamp
 * @param {string} [comment]
 * @returns {PropertyDecorator}
 */
export declare function TimestampColumn(timeWhen?: timeWhen, comment?: string): PropertyDecorator;
export declare type relationType = "HASONE" | "HASMANY" | "MANYTOMANY";
/**
 * Relations mapping
 *
 * @export
 * @param {Function} clazz
 * @param {relationType} type
 * @param {string} foreignKey
 * @param {string} relationKey
 * @returns {PropertyDecorator}
 */
export declare function Relations(clazz: Function, type: relationType, foreignKey: string, relationKey: string, map?: Function, field?: string[]): PropertyDecorator;
/**
 * Mark this method to start transaction execution. In Thinkkoa and Koatty.
 *
 * @export
 * @param {string} modelInstanceName Instantiated ORM model attribute members of the current class.
 * * exp:
 * * @Transactional("userModel")
 * * await this.userModel.add({name: "test"});
 * * await this.userModel.where({id: 1}).update({status: 1});
 * * const tsx = await this.userModel.getInstance();
 * * await this.roleModel.setInstance(tsx).where({id: 1}).update(name: "test");
 * @returns {MethodDecorator}
 */
export declare function Transactional(modelInstanceName: string): MethodDecorator;
