"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @ author: richen
 * @ copyright: Copyright (c) - <richenlin(at)gmail.com>
 * @ license: MIT
 * @ version: 2020-02-27 12:53:07
 */
// tslint:disable-next-line: no-import-side-effect
require("reflect-metadata");
const Validator_1 = require("./Validator");
const think_validtion_1 = require("think_validtion");
const liteq = require('liteq');
const helper = liteq.helper;
var think_validtion_2 = require("think_validtion");
exports.ClassValidator = think_validtion_2.ClassValidator;
exports.FunctionValidator = think_validtion_2.FunctionValidator;
exports.IsDefined = think_validtion_2.IsDefined;
exports.IsCnName = think_validtion_2.IsCnName;
exports.IsIdNumber = think_validtion_2.IsIdNumber;
exports.IsZipCode = think_validtion_2.IsZipCode;
exports.IsMobile = think_validtion_2.IsMobile;
exports.IsPlateNumber = think_validtion_2.IsPlateNumber;
exports.IsEmail = think_validtion_2.IsEmail;
exports.IsIP = think_validtion_2.IsIP;
exports.IsPhoneNumber = think_validtion_2.IsPhoneNumber;
exports.IsUrl = think_validtion_2.IsUrl;
exports.IsHash = think_validtion_2.IsHash;
exports.IsNotEmpty = think_validtion_2.IsNotEmpty;
exports.Equals = think_validtion_2.Equals;
exports.NotEquals = think_validtion_2.NotEquals;
exports.Contains = think_validtion_2.Contains;
exports.IsIn = think_validtion_2.IsIn;
exports.IsNotIn = think_validtion_2.IsNotIn;
exports.IsDate = think_validtion_2.IsDate;
exports.Min = think_validtion_2.Min;
exports.Max = think_validtion_2.Max;
exports.Length = think_validtion_2.Length;
/**
 * Dynamically add methods for target class types
 *
 * @param {Function} clazz
 * @param {string} protoName
 */
function defineNewProperty(clazz, protoName) {
    const oldMethod = Reflect.get(clazz.prototype, protoName);
    Reflect.defineProperty(clazz.prototype, protoName, {
        writable: true,
        value: async function fn(data, options) {
            if (oldMethod) {
                // tslint:disable-next-line: no-invalid-this
                data = await Promise.resolve(Reflect.apply(oldMethod, this, [data, options]));
            }
            // tslint:disable-next-line: no-invalid-this
            clazz.prototype.config = this.config || {};
            // if (helper.isEmpty(data)) {
            //     return {};
            // } else 
            if (helper.isArray(data)) {
                const result = [];
                // tslint:disable-next-line: prefer-for-of
                for (const i of data) {
                    // tslint:disable-next-line: no-invalid-this
                    result.push(Validator_1.Validator(clazz, this.fields, i, protoName));
                }
                data = await Promise.all(result);
            }
            else {
                // tslint:disable-next-line: no-invalid-this
                data = await Validator_1.Validator(clazz, this.fields, data, protoName);
            }
            return data;
        }
    });
}
/**
 * Check the colum name
 *
 * @param {string} name
 * @returns
 */
function checkColumn(name) {
    const deArr = ['pk', 'config', 'options', 'instance', 'modelName', 'tableName', 'relations', 'relationShip'];
    if (deArr.includes(name)) {
        throw Error(`The column name '${name}' cannot be a reserved keyword.`);
    }
    return;
}
/**
 * Cover designType to knex type
 *
 * @param {string} designType
 * @param {number} size
 * @returns
 */
function switchType(designType, size) {
    let ctype = "";
    switch (designType) {
        case "Number":
            ctype = "integer";
            if (size > 11) {
                ctype = 'bigInteger';
            }
            break;
        case "Array":
            ctype = "array";
            break;
        case "Object":
            ctype = "json";
            break;
        case "Boolean":
            ctype = "boolean";
            break;
        case "String":
            if (size > 255) {
                ctype = "text";
            }
            else {
                ctype = "string";
            }
            break;
        default:
            ctype = "string";
            break;
    }
    return ctype;
}
/**
 * This decorator is used to mark classes that will be an entity (table or document depend on database type).
 * Database schema will be created for all classes decorated with it, and Repository can be retrieved and used for it.
 *
 * @export
 * @param {string} [identifier]
 * @returns {ClassDecorator}
 */
function Entity(identifier) {
    return (target) => {
        identifier = identifier || target.name;
        if (!target.prototype.modelName) {
            Reflect.defineProperty(target.prototype, "modelName", {
                value: identifier,
                writable: true,
                configurable: true,
                enumerable: true
            });
        }
        //
        defineNewProperty(target, "_beforeAdd");
        defineNewProperty(target, "_beforeUpdate");
    };
}
exports.Entity = Entity;
/**
 * Column decorator is used to mark a specific class property as a table column.
 * Only properties decorated with this decorator will be persisted to the database when entity be saved.
 * Primary columns also creates a PRIMARY KEY for this column in a db.
 *
 * @export
 * @param {number} [size=11]
 * @param {boolean} [auto=true]
 * @param {string} [comment]
 * @returns {PropertyDecorator}
 */
function PrimaryColumn(size = 11, auto = true, comment) {
    return (target, propertyKey) => {
        //Check the colum name
        checkColumn(propertyKey);
        if (!target.fields) {
            Reflect.defineProperty(target, "fields", {
                value: {},
                writable: true,
                configurable: true,
                enumerable: true
            });
        }
        const designType = Reflect.getMetadata("design:type", target, propertyKey);
        const ctype = switchType(designType.name, size);
        Reflect.defineProperty(target.fields, propertyKey, {
            value: {
                type: ctype,
                size,
                pk: true,
                auto,
                comment
            },
            writable: true,
            configurable: true,
            enumerable: true
        });
        Reflect.defineProperty(target, 'pk', {
            value: propertyKey,
            writable: true,
            configurable: true,
            enumerable: true
        });
        // Set property as included in the process of transformation.
        think_validtion_1.setExpose(target, propertyKey);
    };
}
exports.PrimaryColumn = PrimaryColumn;
/**
 * Column decorator is used to mark a specific class property as a table column.
 *
 * @export
 * @param {number} [size]
 * @param {*} [defaultValue]
 * @param {boolean} [index=false]
 * @param {boolean} [unique=false]
 * @param {string} [comment]
 * @returns {PropertyDecorator}
 */
function Column(size, defaultValue, index = false, unique = false, comment) {
    return (target, propertyKey) => {
        //Check the colum name
        checkColumn(propertyKey);
        if (!target.fields) {
            Reflect.defineProperty(target, "fields", {
                value: {},
                writable: true,
                configurable: true,
                enumerable: true
            });
        }
        const designType = Reflect.getMetadata("design:type", target, propertyKey);
        const ctype = switchType(designType.name, size);
        const values = {
            type: ctype,
            comment
        };
        if (size > 0) {
            values.size = size;
        }
        if (defaultValue === null) {
            values.isnull = true;
        }
        if (defaultValue !== null && defaultValue !== undefined) {
            values.defaults = defaultValue;
        }
        if (index) {
            values.index = index;
        }
        if (unique) {
            values.unique = unique;
        }
        Reflect.defineProperty(target.fields, propertyKey, {
            value: values,
            writable: true,
            configurable: true,
            enumerable: true
        });
        // Set property as included in the process of transformation.
        think_validtion_1.setExpose(target, propertyKey);
    };
}
exports.Column = Column;
/**
 * This column will store a creation Unix timestamp of the inserted/updated object.
 *
 * @export
 * @param {timeWhen} [timeWhen="All"] When to execute timestamp
 * @param {string} [comment]
 * @returns {PropertyDecorator}
 */
function TimestampColumn(timeWhen = "All", comment) {
    return (target, propertyKey) => {
        //Check the colum name
        checkColumn(propertyKey);
        if (!target.fields) {
            Reflect.defineProperty(target, "fields", {
                value: {},
                writable: true,
                configurable: true,
                enumerable: true
            });
        }
        const designType = Reflect.getMetadata("design:type", target, propertyKey);
        if (designType && designType.name !== "Number") {
            throw Error("TimestampColumn's type must be `number`");
        }
        Reflect.defineProperty(target.fields, propertyKey, {
            value: {
                type: "integer",
                size: 11,
                defaults: helper.datetime,
                isnull: true,
                comment,
                when: timeWhen
            },
            writable: true,
            configurable: true,
            enumerable: true
        });
        // Set property as included in the process of transformation.
        think_validtion_1.setExpose(target, propertyKey);
    };
}
exports.TimestampColumn = TimestampColumn;
/**
 * Relations mapping
 *
 * @export
 * @param {Function} clazz
 * @param {relationType} type
 * @param {string} foreignKey
 * @param {string} relationKey
 * @returns {PropertyDecorator}
 */
function Relations(clazz, type, foreignKey, relationKey, map, field) {
    return (target, propertyKey) => {
        if (!target.relations) {
            Reflect.defineProperty(target, "relations", {
                value: {},
                writable: true,
                configurable: true,
                enumerable: true
            });
        }
        const values = {
            type,
            model: clazz,
            fkey: foreignKey,
            rkey: relationKey
        };
        if (type === "MANYTOMANY") {
            if (helper.isEmpty(map)) {
                throw Error(`Missing map table parameter`);
            }
            values.map = map;
        }
        if (field) {
            values.field = field;
        }
        Reflect.defineProperty(target.relations, propertyKey, {
            value: values,
            writable: true,
            configurable: true,
            enumerable: true
        });
    };
}
exports.Relations = Relations;
/**
 * Mark this method to start transaction execution. In Thinkkoa and Koatty.
 *
 * @export
 * @param {string} modelInstanceName Instantiated ORM model attribute members of the current class.
 * * exp:
 * * @Transactional("userModel")
 * * await this.userModel.add({name: "test"});
 * * await this.userModel.where({id: 1}).update({status: 1});
 * * const tsx = await this.userModel.getInstance();
 * * await this.roleModel.setInstance(tsx).where({id: 1}).update(name: "test");
 * @returns {MethodDecorator}
 */
function Transactional(modelInstanceName) {
    return (target, methodName, descriptor) => {
        const { value, configurable, enumerable } = descriptor;
        descriptor = {
            configurable,
            enumerable,
            writable: true,
            value: async function trans(...props) {
                // tslint:disable-next-line: no-invalid-this
                const modelCls = this[modelInstanceName];
                if (helper.isEmpty(modelCls) || helper.isEmpty(modelCls.config)) {
                    return Promise.reject("Model instance is invalid.");
                }
                return modelCls.transaction((tsx) => {
                    // tslint:disable-next-line: no-invalid-this
                    return value.apply(this, props);
                });
            }
        };
        return descriptor;
    };
}
exports.Transactional = Transactional;
//# sourceMappingURL=Decorators.js.map